/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "available.hh"
#include "chan_list.hh"
#include "loc_list.hh"
#include <iostream>

using namespace sends;
using namespace std;

//======================================  Constructor
available::available(void) {
}

//======================================  Clear the list
void
available::clear(void) {
    _vect.clear();
}

//======================================  Set a list from a channel source list
void 
available::set_list(const chan_entry& ce, const std::string& source) {
    size_t N = ce.getNSource();
    for (size_t i=0; i<N; i++) {
	const chan_entry::source_entry& se = ce.source(i);
	if (se.name() == source) _vect.push_back(se.refEpoch());
    }
    sort();
}

//======================================  Set from a source location vector
void 
available::set_list(const loc_list& sl) {
    size_t N = sl.size();
    for (size_t i=0; i<N; i++) {
	_vect.push_back(epoch(sl[i].start(), sl[i].stop()));
    }
}

//======================================  Output operator
std::ostream& 
operator<<(std::ostream& out, const available& av) {
    size_t N = av.size();
    if (N) {
	out << av[0];
	for (size_t i=1; i<N; i++) {
	    out << ", " << av[i];
	}
    }
    return out;
}

//======================================  And of two lists
available& 
available::operator&=(const available& x) {
    size_t n1 = size();
    size_t n2 = x.size();
    if (!n1 || !n2) {
	clear();
    } else {
	epik_vect temp;
	temp.reserve(n1+n2);
	for (size_t i=0,j=0; i<n1 && j<n2; ) {
	    epoch epik(_vect[i]);
	    if (epik.overlapx(x._vect[j]))  {
		epik &= x._vect[j];
		temp.push_back(epik);
	    }
	    if (_vect[i].end() < x._vect[j].end()) i++;
	    else                                   j++;
	}
	_vect.swap(temp);
    }
    return *this;
}

//======================================  And of list and epoch
available& 
available::operator&=(const epoch& x) {
    size_t N = size();
    if (N && !x.all()) {
	size_t j=0;
	for (size_t i=0; i<N; i++) {
	    if (_vect[i].overlapx(x))  {
		if (i != j) _vect[j] = _vect[i];
		_vect[j++] &= x;
	    }
	}
	if (j != N) _vect.erase(_vect.begin()+j, _vect.begin()+N);
    }
    return *this;
}

//======================================  Or of two lists.
available& 
available::operator|=(const available& x) {
    //----------------------------------  Nothing to do if x is empty.
    if (x.empty()) {
	return *this;
    }

    //----------------------------------  Result is x if this is empty
    else if (empty()) {
	_vect = x._vect;
    } 

    //----------------------------------  Calculate the combo
    else {
	size_t n1 = size();
	size_t n2 = x.size();
	epik_vect temp;
	temp.reserve(n1+n2);
	for (size_t i=0,j=0; i<n1 || j<n2; ) {
	    //--------------------------  Only x left
	    if (i >= n1) {
		if (temp.back().overlap(x._vect[j])) temp.back() |= x._vect[j];
		else if (!x._vect[j].none())         temp.push_back(x._vect[j]);
		j++;
	    }

	    //--------------------------  Only this left
	    if (j >= n2) {
		if (temp.back().overlap(_vect[i])) temp.back() |= _vect[i];
		else if (!_vect[i].none())         temp.push_back(_vect[i]);
		i++;
	    }

	    //--------------------------  Process first
	    if (x._vect[j] < _vect[i]) {
		if (!temp.empty() && temp.back().overlap(x._vect[j])) {
		    temp.back() |= x._vect[j++];
		} else {
		    if (!x._vect[j].none()) temp.push_back(x._vect[j]);
		    j++;
		}
	    } else {
		if (!temp.empty() && temp.back().overlap(_vect[i])) {
		    temp.back() |= _vect[i++];
		} else {
		    if (!_vect[i].none()) temp.push_back(_vect[i]);
		    i++;
		}
	    }
	}
	_vect.swap(temp);
    }
    return *this;
}

//======================================  Sort the vector
void
available::sort(void) {
    _vect.sort();
}

//======================================  Output listst for specified source
void 
available::src_list(const std::string& src, std::ostream& out) const {
    size_t N = size();
    for (size_t i=0; i<N; i++) {
	if (i) out << " ";
	out << src << ":";
	_vect[i].display(out, epoch::st_trans);
    }
}
